import datetime
import time
from unittest.mock import patch

import pytest

from geopy.adapters import get_retry_after


@pytest.mark.parametrize(
    "headers, expected_retry_after",
    [
        ({}, None),
        ({"retry-after": "42"}, 42),
        ({"retry-after": "Wed, 21 Oct 2015 07:28:44 GMT"}, 43),
        ({"retry-after": "Wed, 21 Oct 2015 06:28:44 GMT"}, 0),
        ({"retry-after": "Wed"}, None),
    ],
)
def test_get_retry_after(headers, expected_retry_after):
    current_time = datetime.datetime(
        2015, 10, 21, 7, 28, 1, tzinfo=datetime.timezone.utc
    ).timestamp()
    with patch.object(time, "time", return_value=current_time):
        assert expected_retry_after == get_retry_after(headers)



import pytest

from geopy import exc
from geopy.geocoders import Woosmap
from geopy.point import Point
from test.geocoders.util import BaseTestGeocoder, env


class TestWoosmap(BaseTestGeocoder):
    @classmethod
    def make_geocoder(cls, **kwargs):
        return Woosmap(api_key=env['WOOSMAP_KEY'], **kwargs)

    async def test_user_agent_custom(self):
        geocoder = Woosmap(
            api_key='DUMMYPRIVATEKEY1234',
            user_agent='my_user_agent/1.0'
        )
        assert geocoder.headers['User-Agent'] == 'my_user_agent/1.0'

    async def test_check_status(self):
        def make_status(status):
            return {
                "status": status,
            }

        assert self.geocoder._check_status(make_status("ZERO_RESULTS")) is None
        with pytest.raises(exc.GeocoderQueryError):
            self.geocoder._check_status(make_status("INVALID_REQUEST"))
        with pytest.raises(exc.GeocoderQueryError):
            self.geocoder._check_status(make_status("REQUEST_DENIED"))
        with pytest.raises(exc.GeocoderServiceError):
            self.geocoder._check_status(make_status("UNKNOWN_ERROR"))
        with pytest.raises(exc.GeocoderServiceError):
            self.geocoder._check_status(make_status("DUMMY_ERROR"))

    async def test_format_components_param(self):
        f = self.geocoder._format_components_param
        assert f({}) == ''
        assert f([]) == ''

        assert f([('country', 'FR')]) == 'country:FR'
        output = f([
            ('country', 'DE'),
            ('country', 'GB'),
            ('country', 'US')
        ])
        assert (
            output ==
            'country:DE|country:GB|country:US'
        )

        with pytest.raises(ValueError):
            f(None)

        with pytest.raises(ValueError):
            f('country:DE|country:FR')

    async def test_geocode(self):
        await self.geocode_run(
            {"query": "12 Oxford Street, London"},
            {"latitude": 51.51652, "longitude": -0.131},
        )

    async def test_geocode_with_components(self):
        await self.geocode_run(
            {
                "query": "paris",
                "components": {
                    "country": "FR"
                }
            },
            {"latitude": 48.85717, "longitude": 2.3414},
        )

        await self.geocode_run(
            {
                "query": "paris",
                "components": [
                    ('country', 'US')
                ]
            },
            {"latitude": 33.66128, "longitude": -95.56356},
        )

    async def test_geocode_with_location_bias(self):
        await self.geocode_run(
            {
                "query": "paris",  # Texas USA
                "location": (33.65919, -95.55486)
            },
            {"latitude": 33.66128, "longitude": -95.56356},
        )

    async def test_geocode_with_language(self):
        result = await self.geocode_run(
            {"query": "12 Oxford Street, London", "language": "fr"},
            {}
        )
        assert "Royaume-Uni" in result.address

        result = await self.geocode_run(
            {"query": "12 Oxford Street, London", "language": "en"},
            {}
        )
        assert "United Kingdom" in result.address

    async def test_geocode_with_country_code_format(self):
        result = await self.geocode_run(
            {"query": "12 Oxford Street, London", "country_code_format": "alpha3"},
            {}
        )
        country_component = [comp for comp in result.raw["address_components"] if
                             'country' in comp['types']][0]
        assert country_component.get('short_name') == "GBR"

        result = await self.geocode_run(
            {"query": "12 Oxford Street, London", "country_code_format": "alpha2"},
            {}
        )
        country_component = [comp for comp in result.raw["address_components"] if
                             'country' in comp['types']][0]
        assert country_component.get('short_name') == "GB"

    async def test_geocode_limit(self):
        result = await self.geocode_run(
            {
                "query": "oxford street",
                "limit": 5,
                "exactly_one": False
            },
            {}
        )
        assert len(result) == 5

    async def test_reverse(self):
        london_point = Point(51.50940214, -0.133012)
        await self.reverse_run(
            {"query": london_point},
            {"latitude": 51.50940214, "longitude": -0.133012},
        )

    async def test_reverse_with_language(self):
        london_point = Point(51.50940214, -0.133012)
        result = await self.reverse_run(
            {"query": london_point, "language": "fr"},
            {}
        )
        assert "Royaume-Uni" in result.address

        result = await self.reverse_run(
            {"query": london_point, "language": "en"},
            {}
        )
        assert "United Kingdom" in result.address

    async def test_reverse_with_country_code_format(self):
        london_point = Point(51.50940214, -0.133012)
        result = await self.reverse_run(
            {"query": london_point, "country_code_format": "alpha3"},
            {}
        )
        country_component = [comp for comp in result.raw["address_components"] if
                             'country' in comp['types']][0]
        assert country_component.get('short_name') == "GBR"

        result = await self.reverse_run(
            {"query": london_point, "country_code_format": "alpha2"},
            {}
        )
        country_component = [comp for comp in result.raw["address_components"] if
                             'country' in comp['types']][0]
        assert country_component.get('short_name') == "GB"

    async def test_reverse_limit(self):
        london_point = Point(51.50940214, -0.133012)
        result = await self.reverse_run(
            {
                "query": london_point,
                "limit": 5,
                "exactly_one": False
            },
            {}
        )
        assert len(result) == 5

    async def test_geocode_empty_result(self):
        await self.geocode_run(
            {"query": "dsadjkasdjasd"},
            {},
            expect_failure=True,
        )

    async def test_reverse_empty_result(self):
        await self.reverse_run(
            {"query": Point(0.05, -0.15)},
            {},
            expect_failure=True,
        )



import pytest

from geopy.exc import (
    GeocoderInsufficientPrivileges,
    GeocoderQuotaExceeded,
    GeocoderRateLimited,
)
from geopy.geocoders import OpenCage
from test.geocoders.util import BaseTestGeocoder, env


class TestUnitOpenCage:

    def test_user_agent_custom(self):
        geocoder = OpenCage(
            api_key='DUMMYKEY1234',
            user_agent='my_user_agent/1.0'
        )
        assert geocoder.headers['User-Agent'] == 'my_user_agent/1.0'


class TestOpenCage(BaseTestGeocoder):

    testing_tokens = {
        # https://opencagedata.com/api#testingkeys
        402: "4372eff77b8343cebfc843eb4da4ddc4",
        403: "2e10e5e828262eb243ec0b54681d699a",
        429: "d6d0f0065f4348a4bdfe4587ba02714b",
    }

    @classmethod
    def make_geocoder(cls, **kwargs):
        return OpenCage(
            api_key=env['OPENCAGE_KEY'],
            timeout=10,
            **kwargs
        )

    async def test_geocode(self):
        await self.geocode_run(
            {"query": "435 north michigan ave, chicago il 60611 usa"},
            {"latitude": 41.890, "longitude": -87.624},
        )

    async def test_geocode_empty_result(self):
        await self.geocode_run(
            {"query": "xqj37"},
            {},
            expect_failure=True
        )

    async def test_bounds(self):
        await self.geocode_run(
            {"query": "moscow",  # Idaho USA
             "bounds": [[50.1, -130.1], [44.1, -100.9]]},
            {"latitude": 46.7323875, "longitude": -117.0001651},
        )

    async def test_country_str(self):
        await self.geocode_run(
            {"query": "kazan",
             "country": 'tr'},
            {"latitude": 40.2317, "longitude": 32.6839},
        )

    async def test_country_list(self):
        await self.geocode_run(
            {"query": "kazan",
             "country": ['cn', 'tr']},
            {"latitude": 40.2317, "longitude": 32.6839},
        )

    async def test_geocode_annotations(self):
        location = await self.geocode_run(
            {"query": "london"},
            {"latitude": 51.5073219, "longitude": -0.1276474},
        )
        assert location.raw['annotations']

        location = await self.geocode_run(
            {"query": "london", "annotations": False},
            {"latitude": 51.5073219, "longitude": -0.1276474},
        )
        assert 'annotations' not in location.raw

    async def test_payment_required_error(self, disable_adapter_retries):
        async with self.inject_geocoder(OpenCage(api_key=self.testing_tokens[402])):
            with pytest.raises(GeocoderQuotaExceeded) as cm:
                await self.geocode_run(
                    {"query": "london"}, {}, skiptest_on_errors=False
                )
            assert cm.type is GeocoderQuotaExceeded
            # urllib: HTTP Error 402: Payment Required
            # others: Non-successful status code 402

    async def test_api_key_disabled_error(self, disable_adapter_retries):
        async with self.inject_geocoder(OpenCage(api_key=self.testing_tokens[403])):
            with pytest.raises(GeocoderInsufficientPrivileges) as cm:
                await self.geocode_run(
                    {"query": "london"}, {}, skiptest_on_errors=False
                )
            assert cm.type is GeocoderInsufficientPrivileges
            # urllib: HTTP Error 403: Forbidden
            # others: Non-successful status code 403

    async def test_rate_limited_error(self, disable_adapter_retries):
        async with self.inject_geocoder(OpenCage(api_key=self.testing_tokens[429])):
            with pytest.raises(GeocoderRateLimited) as cm:
                await self.geocode_run(
                    {"query": "london"}, {}, skiptest_on_errors=False
                )
            assert cm.type is GeocoderRateLimited
            # urllib: HTTP Error 429: Too Many Requests
            # others: Non-successful status code 429



from geopy.geocoders import GeocodeEarth
from test.geocoders.pelias import BaseTestPelias
from test.geocoders.util import env


class TestGeocodeEarth(BaseTestPelias):

    @classmethod
    def make_geocoder(cls, **kwargs):
        return GeocodeEarth(env['GEOCODEEARTH_KEY'],
                            **kwargs)
